/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
/**
 * <p>Provides functionality for Exceptions.</p>
 * <p>Contains the concept of an exception with context i.e. such an exception will contain a map with keys and values.
 * This provides an easy way to pass valuable state information at exception time in useful form to a calling process.</p>
 * <p>Lastly, {@link org.apache.commons.lang3.exception.ExceptionUtils} also contains <code>Throwable</code> manipulation
 * and examination routines.</p>
 *
 * @since 1.0
 * @version $Id: package-info.java 1558546 2014-01-15 19:38:15Z britter $
 */
package org.apache.commons.lang3.exception;
